*****************************************************************************************************
-- MANAGING PRIVILEGES
*****************************************************************************************************

-- A PRIVILEGE IS A RIGHT TO EXECUTE A PARTICULAR TYPE OF SQL STATEMENT 
   OR TO ACCESS ANOTHER USER'S OBJECT. 

-- THERE ARE TWO TYPES OF PRIVILEGES:

1) SYSTEM PRIVILEGES: 
   TO PERFORM A PARTICULAR DATABASE OPERATION OR CLASS 
   OF DATABASE OPERATIONS (SHUTDOWN, MANAGE TABLESPACES, CREATE TABLE, ETC)

2) OBJECT PRIVILEGES: 
   TO PERFORM A PARTICULAR ACTION ON A SPECIFIC OBJECT, SUCH AS A 
   TABLE, VIEW, SEQUENCE, PROCEDURE, FUNCTION, OR PACKAGE (SELECT, INSERT, EXECUTE, UPDATE, DELETE, ETC)

-- TO ALLOW OR DENY FOR BOTH PRIVILEGES WE USE THE DCL (DATA CONTROL LANGUAGE) COMMANDS GRANT AND REVOKE.
-- THE SYNTAX IS SOMETHING LIKE:


GRANT   <privlege> TO <user>
REVOKE  <privlege> FROM <user>

-- LET'S SEE THE SYSTEM PRIVILEGES FIRST.


-- THE ARE OVER 100 DISTINCT SYSTEM PRIVILEGES (SEE Fig1OracleSystemPriv.PNG)
-- THERE IS THE ANY KEYWORD:  SIGNIFIES THAT USERS HAVE THE PRIVILEGE IN ANY SCHEMA
-- EXAMPLE: THE CREATE SESSION PRIVILEGE (USED TO ALLOW USER TO LOG IN THE ORACLE DATABASE)

-- CREATING THE USERS
CREATE USER jonh
IDENTIFIED BY abc123
DEFAULT TABLESPACE USERS
TEMPORARY TABLESPACE TEMP
QUOTA 15m ON USERS
PASSWORD EXPIRE

CREATE USER mary
IDENTIFIED BY abc123
DEFAULT TABLESPACE USERS
TEMPORARY TABLESPACE TEMP
QUOTA 15m ON USERS
PASSWORD EXPIRE

-- GRANTING THE CREATE SESSION PRIVILEGE
GRANT CREATE SESSION TO jonh
GRANT CREATE SESSION TO mary


-- MOST OF THE TIME THE DBA WILL USE ON THE PRIVILEGES FOR DDL COMMANDS (CREATE, ALTER, DROP). 
-- MOST USED PRIVILEGES IN Fig2MostUsed.png
-- NOTE: THE IS THE SELECT ANY, UPDATE ANY, AND DELETE ANY SYSTEM PRIVILEGES MUST BE USED WITH CAUTION!

-- CHECK THE DOCUMENTATION FOR MORE PRIVILEGES.
-- EXAMPLE:


-- IN A ADMIN LOGIN SESSION:
GRANT CREATE TABLE TO jonh;

-- IN A jonh LOGIN SESSION:
CREATE TABLE T_JONH
( 
ID INT
);



-- IN A ADMIN OR jonh SESSION
SELECT * FROM jonh.T_JONH;

-- THE WITH ADMIN OPTION ALLOWS THE PROPAGATION OF THE PERMISSION. EXAMPLE:
-- IN A ADMIN LOGIN SESSION:

GRANT CREATE TABLE TO jonh WITH ADMIN OPTION;

-- IN A jonh LOGIN SESSION:
GRANT CREATE TABLE TO MARY

-- IN A mary LOGIN SESSION:
CREATE TABLE T_MARY
( 
ID INT
);


-- IN A ADMIN OR mary SESSION
SELECT * FROM mary.T_MARY;

-- YOU CAN ALSO REMOVE PERMISSIONS WITH THE REVOKE COMMAND. EXAMPLE:

-- IN A ADMIN LOGIN SESSION:
REVOKE CREATE TABLE FROM jonh;

-- IN A jonh LOGIN SESSION THE FOLLOWING COMMAND WILL FAIL:
CREATE TABLE T_JONH_ANOTHER
( 
ID INT
);


-- OBJECT PRIVILEGES: THE MOST COMMON OBJECT PRIVILEGES x OBJECTS IN Fig3ObjPriv.png
-- THE SYNTAX FOR THE GRANT AND DENY IS A LITTLE BIT DIFFERENT:

GRANT   <privilege> ON <object> TO <user>
REVOKE  <privilege> ON <object> FROM <user>

-- THE 'WITH GRANT OPTION' ALLOW THE PROPAGATION OF THE PRIVILEGE
-- EXAMPLES:

-- IN A ADMIN OR hr SESSION
GRANT SELECT ON hr.countries TO jonh;
GRANT SELECT ON hr.countries TO mary;

-- IN jonh or mary SESSION
SELECT * FROM hr.countries;

-- IN A ADMIN OR hr SESSION
REVOKE SELECT ON hr.countries FROM mary;

-- IN mary SESSION the following commando will fail
SELECT * FROM hr.countries;

-- THERE ARE SOME DATA DICTIONARY VIEWS TO CHECK PRIVILEGES INFO:

DBA_SYS_PRIVS -> LISTS SYSTEM PRIVILEGES GRANTED TO USERS AND ROLES
SESSION_PRIVS -> LISTS THE PRIVILEGES THAT ARE CURRENTLY AVAILABLE TO THE USER
DBA_TAB_PRIVS -> LISTS ALL GRANTS ON ALL OBJECTS IN THE DATABASE
DBA_COL_PRIVS -> DESCRIBES ALL OBJECT COLUMN GRANTS IN THE DATABASE.

SELECT * FROM DBA_SYS_PRIVS 

SELECT * FROM DBA_TAB_PRIVS

